const async   = require('async');
const expect  = require('chai').expect;
const exists  = require('fs').existsSync;
const exec    = require('child_process').exec;
const join    = require('path').join;
const request = require('supertest');
const app     = require('../lib');
const dirname = join(__dirname, '..');

describe('C8 - POST /publish', function() {
  before(function(done) {
    clear(function() {
      publish('component/each', done);
    });
  });

  it('creates public and temporary files', function() {
    expect(exists(join(dirname, 'public/component')));
    expect(exists(join(dirname, 'public/component/each.js')));
    expect(exists(join(dirname, 'public/component/each.css')));
    expect(exists(join(dirname, 'public/component/each.min.js')));
    expect(exists(join(dirname, 'public/component/each.min.css')));
    expect(exists(join(dirname, 'public/component/each.json')));
    expect(exists(join(dirname, 'tmp/component/each')));
    expect(exists(join(dirname, 'tmp/component/component-each')));
  });
});

describe('C8 - GET /concat', function() {
  before(function(done) {
    clear(function() {
      async.parallel([
        function(cb) { publish('component/type', cb) },
        function(cb) { publish('component/calendar', cb) },
        function(cb) { publish('ask11/indexed', cb) }
      ], done);
    });
  });

  it('concats 3 repos', function(done) {
    request(app)
      .get('/concat.js?repos=component/type,component/calendar,ask11/indexed')
      .expect(200)
      .end(function(err, res) {
        expect(res.text).contain('indexed/index.js');
        expect(res.text).contain('type/index.js');
        expect(res.text).contain('calendar/index.js');
        done(err);
      });
  });

  it('ignores not existing repos', function(done) {
    request(app)
      .get('/concat.js?repos=component/type,component/dom')
      .expect(200)
      .end(function(err, res) {
        expect(res.text).contain('type/index.js');
        expect(res.text).not.contain('dom/index.js');
        done(err);
      });
  });

  it('returns css files too', function(done) {
    request(app)
      .get('/concat.css?repos=component/type,component/calendar,ask11/indexed')
      .expect(200)
      .end(function(err, res) {
        expect(res.text).contain('.calendar-table');
        done(err);
      });
  });

  it('returns nothing when files not found', function(done) {
    request(app)
      .get('/concat.css?repos=component/type,ask11/indexed')
      .expect(200)
      .end(function(err, res) {
        expect(res.text).equal('');
        done(err);
      });
  });
});

function publish(repo, cb) {
  request(app)
    .post('/publish')
    .send({ repo: repo })
    .expect(204)
    .end(cb);
}

function clear(done) {
  async.parallel([
    function(cb) { exec('rm -rf tmp', cb) },
    function(cb) { exec('rm -rf public/component public/ask11', cb) }
  ], done);
}
